import 'package:code_id_flutter/code_interfaces/storage/i_storage.dart';
import 'package:dio/dio.dart';

enum AuthType { basic, bearer, custom }

extension AuthTypeX on AuthType {
  String get type {
    switch (this) {
      case AuthType.basic:
        return 'Basic';

      default:
        return 'Bearer';
    }
  }
}

class AuthInterceptor extends Interceptor {
  // final String? token;
  final IStorage storage;
  final String authKey;
  final AuthType authType;
  // final String
  AuthInterceptor({
    required this.storage,
    required this.authKey,
    this.authType = AuthType.bearer,
  });

  @override
  void onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    if (storage.isStorageOpen()) {
      final String? security = storage.getString(key: authKey);

      Map<String, dynamic> headers = options.headers;
      if (security != null) {
        headers.addAll({"Authorization": "Bearer $security"});
      }
      options.headers = headers;
    }

    super.onRequest(options, handler);
  }
}
